from django.contrib.gis.db.models.query import GeoQuerySet


class LiveGeoQuerySet(GeoQuerySet):

    def delete(self):
        # Override Django's built-in default.
        self.soft_delete()

    def soft_delete(self):
        self.update(live=False)

    def undelete(self):
        self.update(live=True)

    def hard_delete(self):
        # Default Django behavior.
        super(LiveGeoQuerySet, self).delete()

    def live(self):
        return self.filter(live=True)

    def non_dead(self):
        return self.live()

    def dead(self):
        return self.filter(live__isnull=True)



from django.db import models
from .querysets import LiveGeoQuerySet


class LiveGeoManagerBase(models.Manager):
    # To also use this manager for FK lookups, see
    # https://docs.djangoproject.com/en/1.6/topics/db/managers/#manager-types.

    def __init__(self, include_soft_deleted=False, *args, **kwargs):
        self.include_soft_deleted = include_soft_deleted
        super(LiveGeoManagerBase, self).__init__(*args, **kwargs)  # pylint: disable=super-on-old-class

    def get_queryset(self):
        qs = super(LiveGeoManagerBase, self).get_queryset()  # pylint: disable=super-on-old-class
        if not self.include_soft_deleted:
            return qs.live()
        return qs


LiveGeoManager = LiveGeoManagerBase.from_queryset(LiveGeoQuerySet)



from django.contrib.gis.db.models import Model as GeoModel

from ..fields import LiveField
from .managers import LiveGeoManager


class LiveGeoModel(GeoModel):
    """GeoModel support for soft-deleting using LiveField.

    LiveModel overrides Model.delete() to provide soft-deletion via
    a LiveField. `.delete()` updates `Model.live` to `True`. Normal
    deletion can performed usign `Model.hard_delete()`.
    """

    live = LiveField()

    objects = LiveGeoManager()
    all_objects = LiveGeoManager(include_soft_deleted=True)

    class Meta:
        abstract = True

    def delete(self, *args, **kwargs):
        self.live = False
        self.save()

    def hard_delete(self):  # pylint: disable=super-on-old-class
        super(LiveGeoModel, self).delete()

    def undelete(self, *args, **kwargs):
        self.live = True
        self.save()
